using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using MVR.FileManagementSecure;
using UnityEngine.UI;
using System.Runtime.InteropServices;
using static OVRInput;
using UnityEngine.UI.Extensions;

namespace JayJayWon
{
    public class ResourceActionsUI
    {
        private GameObject actionsUIBoxGO;
        private GameObject tabsUIBoxGO;
        private BrowserUI parentBrowser;

        private Dictionary<int, ResourceActionUIBase> resourceActionUITabs = new Dictionary<int, ResourceActionUIBase>();

        private List<UIDynamicButton> resourceActionTabButtons = new List<UIDynamicButton>();

        public int currentActionUITabType = ActionUITabType.primaryActionTab;

        public const float resourceActionTabsHeight = 15f;
        const float resourceActionTabsWidth = 50f;

        public ResourceActionUIBase currentActionUITab
        {
            get
            {
                List<int> actionChoices = GetActionUITabChoices();
                if (!actionChoices.Contains(currentActionUITabType))
                {
                    if (resourceActionUITabs.ContainsKey(currentActionUITabType)) resourceActionUITabs[currentActionUITabType].ClearUI();
                    currentActionUITabType = actionChoices[0];
                    RefreshActionTabs();
                }
                if (resourceActionUITabs.ContainsKey(currentActionUITabType)) return resourceActionUITabs[currentActionUITabType];
                return CreateActionTab(currentActionUITabType);
            }
        }
        public ResourcePrimaryActionUI primaryActionUITab
        {
            get
            {
                List<int> actionChoices = GetActionUITabChoices();

                if (resourceActionUITabs.ContainsKey(ActionUITabType.primaryActionTab)) return (ResourcePrimaryActionUI)resourceActionUITabs[ActionUITabType.primaryActionTab];
                return (ResourcePrimaryActionUI)CreateActionTab(ActionUITabType.primaryActionTab);
            }
        }
        private void RefreshActionTabs()
        {
            var actionTabTypes = GetActionUITabChoices();
            int currentTabButtonCount = resourceActionTabButtons.Count;
            int requiredTabButtonCount = actionTabTypes.Count;

            if (currentTabButtonCount < requiredTabButtonCount)
            {
                for (int i = currentTabButtonCount; i < requiredTabButtonCount; i++) resourceActionTabButtons.Add(CreateResourceActionTabButton(i));
            }
            else if (currentTabButtonCount > requiredTabButtonCount)
            {
                for (int i = requiredTabButtonCount; i < currentTabButtonCount; i++)
                {
                    GameObject.Destroy(resourceActionTabButtons[i].gameObject);
                    resourceActionTabButtons.RemoveRange(requiredTabButtonCount, (currentTabButtonCount - requiredTabButtonCount));
                }
            }
            int index = 0;
            foreach (var tab in resourceActionTabButtons)
            {
                tab.label = ActionUITabType.GetTabLabel(actionTabTypes[index]);
                int actionTabType = actionTabTypes[index];
                if (actionTabType == currentActionUITabType)
                {
                    tab.buttonColor = BrowserUI.darkBlueGrey;
                    tab.textColor = Color.white;
                }
                else
                {
                    tab.buttonColor = BrowserUI.lightGrey;
                    tab.textColor = Color.black;
                }
                var buttonUID = resourceActionTabButtons[index];
                buttonUID.button.onClick.RemoveAllListeners();
                buttonUID.button.onClick.AddListener(delegate {
                    TabSelected(actionTabType);
                });
                AssignTabHelpText(buttonUID, actionTabType);
                index++;
            }
        }

        private void AssignTabHelpText(UIDynamicButton buttonUID, int actionTabType)
        {
            string helpText = "";

            if (actionTabType == ActionUITabType.primaryActionTab) helpText = "Perform primary actions on the selected resource. Where multiple resource are selected, the primary action will be performed on the last selected resource.";
            if (actionTabType == ActionUITabType.tagsTab) helpText = "Assign tags to the selected resources.";
            if (actionTabType == ActionUITabType.positionTab) helpText = "Adjust the position of Scene Atoms";
            if (actionTabType == ActionUITabType.infoTab) helpText = "View and modify info and meta data assoicated with the selected resource(s)";
            parentBrowser.ApplyHelp(buttonUID, helpText);
        }

        private void TabSelected(int tabType)
        {
            if (currentActionUITabType != tabType)
            {
                currentActionUITab.ClearUI();
                currentActionUITabType = tabType;
                RefreshUI();
            }
        }
        private UIDynamicButton CreateResourceActionTabButton(int i)
        {
            UIDynamicButton newTabButton = UI.CreateUIButton(tabsUIBoxGO, resourceActionTabsWidth, resourceActionTabsHeight, i.ToString(), 20, i * (resourceActionTabsWidth+1), anchorPreset: AnchorPresets.bottomLeft);
            newTabButton.buttonImage.sprite = null;
            return newTabButton;
        }

        public void RefreshUI()
        {
            RefreshActionTabs();
            currentActionUITab.RefreshUI();
        }

        public ResourceActionsUI(BrowserUI _parentBrowser, GameObject _parentBoxGO)
        {
            parentBrowser = _parentBrowser;

            tabsUIBoxGO = UI.AddChildGOImage(parentGO: _parentBoxGO,
                color: Color.grey,
                anchorPreset: AnchorPresets.topMiddle,
                horizontalSize: 640f,
                verticalSize: resourceActionTabsHeight,
                anchoredPositionOffset: new Vector2(0f, 0f));

            actionsUIBoxGO = UI.AddChildGOImage(parentGO: _parentBoxGO,
                color: BrowserUI.darkBlueGrey,
                anchorPreset: AnchorPresets.bottomMiddle,
                horizontalSize: 640f,
                verticalSize: BrowserUI.resourceActionBoxHeight - resourceActionTabsHeight,
                anchoredPositionOffset: new Vector2(0f, 0f));


            ResourcePositionActionUI.existingAtomPosition = Globals.mvrScript.gameObject.AddComponent<AtomPositionRotationSync>();

            ResourcePositionActionUI.newAtomPosition = Globals.mvrScript.gameObject.AddComponent<RelativeAtomPositionRotation>();

            ResourcePositionActionUI.newAtomPosition.InitNoCallback();
        }

        public void AtomNameUpdated(string oldName, string newName)
        {
            foreach (var kvp in resourceActionUITabs) kvp.Value.AtomNameUpdated(oldName, newName);
        }

        public void OnAtomAdded(Atom atom)
        {
            primaryActionUITab.targetAtomJSMultiEnum.val = atom.name;
            if (BrowserAssist.mainBrowserUI.isVisible) primaryActionUITab.RefreshTargetAtomValue();

        }

        public void OnAtomRemoved(Atom atom)
        {
            if (BrowserAssist.mainBrowserUI.isVisible) primaryActionUITab.RefreshTargetAtomValue();

            foreach (var kvp in resourceActionUITabs) kvp.Value.OnAtomRemoved(atom);

        }

        private ResourceActionUIBase CreateActionTab(int actionTabType)
        {
            ResourceActionUIBase newActionTab = null;
            if (actionTabType == ActionUITabType.primaryActionTab) newActionTab = new ResourcePrimaryActionUI(parentBrowser, actionsUIBoxGO);
            if (actionTabType == ActionUITabType.tagsTab) newActionTab = new ResourceTagsActionUI(parentBrowser, actionsUIBoxGO);
            if (actionTabType == ActionUITabType.positionTab) newActionTab = new ResourcePositionActionUI(parentBrowser, actionsUIBoxGO, primaryActionUITab);
            if (actionTabType == ActionUITabType.infoTab) newActionTab = new ResourceInfoActionUI(parentBrowser, actionsUIBoxGO);

            if (newActionTab != null)
            {
                if (resourceActionUITabs.ContainsKey(actionTabType)) resourceActionUITabs[actionTabType] = newActionTab;
                else resourceActionUITabs.Add(actionTabType, newActionTab);
            }
            return (newActionTab);
        }

        private List<int> GetActionUITabChoices()
        {
            List<int> choices = new List<int>();

            choices.Add(ActionUITabType.primaryActionTab);
            if (parentBrowser.browserFiltersUI.resourceType != ResourceType.scene && parentBrowser.browserFiltersUI.resourceType!= ResourceType.plugins) choices.Add(ActionUITabType.positionTab);
            choices.Add(ActionUITabType.tagsTab);
            choices.Add(ActionUITabType.infoTab);

            return choices;
        }

    }


    public abstract class ResourceActionUIBase
    {
        protected GameObject parentBoxGO;
        public BrowserUI parentBrowser { get; protected set; }

        protected const float edgeWidth = 2f;

        protected HashSet<UIDynamic> uiElements = new HashSet<UIDynamic>();

        protected HashSet<UIDynamicPopup> uiPopups = new HashSet<UIDynamicPopup>();

        public ResourceActionUIBase(BrowserUI _parentBrowser, GameObject _parentBoxGO)
        {
            parentBrowser = _parentBrowser;
            parentBoxGO = _parentBoxGO;

        }

        public virtual void AtomNameUpdated(string oldName, string newName)
        {
        }
        public virtual void OnAtomRemoved(Atom atom)
        {
        }

        public virtual void RefreshUI()
        {
            ClearUI();
        }

        public virtual void ClearUI()
        {
            foreach (var popup in uiPopups) popup.popup.visible = false;
            foreach (var uiElement in uiElements) if (uiElement!=null) GameObject.Destroy(uiElement.gameObject);
            uiElements.Clear();
        }

        public UIDynamicSlider CreateSlider(GameObject parentGO, JSONStorableFloat jsf, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, float quickButtonHeight = 0f, float labelHeight = 0f, float rangeWidth = 0f, bool hideRange = false, bool hideRangeLabel = false)
        {
            UIDynamicSlider slider = UI.CreateSlider(parentGO, jsf, width, height, label, fontSize, anchorXOffset, anchorYOffset, anchorPreset,quickButtonHeight,labelHeight,rangeWidth, hideRange, hideRangeLabel);
            uiElements.Add(slider);
            return slider;
        }

        public UIDynamicButton CreateUIButton(GameObject parentGO, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none)
        {
            UIDynamicButton button = UI.CreateUIButton(parentGO, width, height, label, fontSize, anchorXOffset, anchorYOffset, anchorPreset);
            uiElements.Add(button);
            return button;
        }

        public UIDynamicPopup CreateScrollablePopup(GameObject parentGO, JSONStorableStringChooser jssc, float width, float height, string label = "", int labelFontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, float popupHeight = 300f, bool popupAbove = false, bool disableSlider=false, int anchorPreset = AnchorPresets.none, int popupFontSize =20)
        {
            UIDynamicPopup popup = UI.CreateScrollablePopup(parentGO, jssc, width, height, label, labelFontSize, anchorXOffset, anchorYOffset, popupHeight,popupAbove, disableSlider, anchorPreset, popupFontSize: popupFontSize);
            uiElements.Add(popup);
            uiPopups.Add(popup);
            return popup;
        }

        public UIDynamicPopup CreatePopup(GameObject parentGO, JSONStorableStringChooser jssc, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool popAbove = false, float labelProportion=0.4f)
        {
            UIDynamicPopup popup = UI.CreatePopup(parentGO, jssc, width, height, label, fontSize, anchorXOffset, anchorYOffset, anchorPreset, popAbove, labelProportion: labelProportion);
            uiElements.Add(popup);
            uiPopups.Add(popup);
            return popup;
        }

        public UIDynamicTextField CreateTextField(GameObject parentGO, JSONStorableString jss, float width, float height, int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none, bool input = false, InputField.LineType inputFieldType = InputField.LineType.MultiLineNewline, string placeHolderText = "", float hScrollBarHeight = 0f, float vScrollBarWidth = 10f, Action<string> textEntryChangeCallback = null)
        {
            UIDynamicTextField textField = UI.CreateTextBox(parentGO, jss, width, height, fontSize, anchorXOffset, anchorYOffset, anchorPreset, input, inputFieldType, placeHolderText, hScrollBarHeight, vScrollBarWidth, textEntryChangeCallback);
            uiElements.Add(textField);
            return textField;
        }

        public UIDynamicToggle CreateToggle(GameObject parentGO, JSONStorableBool jsb, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none)
        {
            UIDynamicToggle toggle = UI.CreateToggle(parentGO, jsb, width, height, label, fontSize, anchorXOffset, anchorYOffset, anchorPreset);
            uiElements.Add(toggle);
            return toggle;
        }
        public TriStateToggle CreateTriStateToggle(GameObject parentGO, JSONStorableBool jsb, float width, float height, string label = "", int fontSize = 20, float anchorXOffset = 0, float anchorYOffset = 0, int anchorPreset = AnchorPresets.none)
        {
            UIDynamicToggle toggle = UI.CreateToggle(parentGO, jsb, width, height, label, fontSize, anchorXOffset, anchorYOffset, anchorPreset);
            uiElements.Add(toggle);
            return new TriStateToggle(jsb, toggle);
        }


    }

    public class ResourcePrimaryActionUI : ResourceActionUIBase
    {
        public JSONStorableMultiEnumStringChooser targetAtomJSMultiEnum;
        public JSONStorableBool mergePresetJSB;
        public JSONStorableBool openPluginUIJSB;
        public JSONStorableBool suppressClothingLoadJSB;
        public JSONStorableBool suppressScaleChangeJSB;

        public JSONStorableBool morphPresetIncludePhysicalJSB;
        public JSONStorableBool morphPresetIncludeAppJSB;

        public JSONStorableBool generalPresetIncludePhysicalJSB;
        public JSONStorableBool generalPresetIncludePoseJSB;
        public JSONStorableBool generalPresetIncludeAppJSB;

        private int resourceType { get { return parentBrowser.browserFiltersUI.resourceType; } }     
        private int resourceCategory { get { return ResourceType.GetResourceCategory(resourceType); } }

        public bool isAtomTargetable { get
            {
                if (resourceCategory == ResourceCategory.preset) return true;
                if (resourceType == ResourceType.plugins && targetAtomJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.topEnumValFlag) return true;
                return false;
            }
        }

        private int resoureTargetCategory { get
            {
                return ResourceType.GetResourceTargetCategory(resourceType);
            } } 

        private ResourceVersionGroupEntry primaryRVGE { get { return parentBrowser.resourceSelectionsUI.GetFirstSelectedRVGE(resourceType); } }

        private UIDynamicPopup targetAtomPopup;
        private UIDynamicButton mainActionButton;
        public bool deleteConfirmationActive { get; set; }=false;

        public ResourcePrimaryActionUI(BrowserUI _parentBrowser, GameObject _parentBoxGO) : base(_parentBrowser, _parentBoxGO)
        {
            targetAtomJSMultiEnum = new JSONStorableMultiEnumStringChooser("targetAtom", null,null, null, null, TargetAtomMainEnumCallBack, CreateNewAtom.enumManifestName,TargetAtomTopEnumCallBack);
            targetAtomJSMultiEnum.label = "Target Atom";
            RefreshTargetAtomPopup(true);
            

            mergePresetJSB = new JSONStorableBool("mergePreset", false);
            openPluginUIJSB = new JSONStorableBool("openPluginUI", false);
            suppressClothingLoadJSB = new JSONStorableBool("suppressClothingLoad", false);
            suppressScaleChangeJSB = new JSONStorableBool("suppressClothingLoad", false);

            morphPresetIncludePhysicalJSB = new JSONStorableBool("morphPresetIncludePhysical", false);
            morphPresetIncludeAppJSB = new JSONStorableBool("morphPresetIncludeApp", true);

            generalPresetIncludePhysicalJSB = new JSONStorableBool("generalPresetIncludePhysical", true);
            generalPresetIncludePoseJSB = new JSONStorableBool("generalPresetIncludePose", true);
            generalPresetIncludeAppJSB = new JSONStorableBool("generalPresetIncludeApp", true);

            
        }

        private void TargetAtomTopEnumCallBack(int i)
        {
            if (parentBrowser.resourceActionsUI.currentActionUITab == this) RefreshUI();
        }
        private void TargetAtomMainEnumCallBack(string i)
        {
            if (parentBrowser.resourceActionsUI.currentActionUITab == this) RefreshUI();
        }

        public void SetNewIcon(ResourceVersionGroupEntry rvge)
        {
            parentBrowser.resourceBrowserUI.SetNewIcon(rvge);
        }

        public override void AtomNameUpdated(string oldName, string newName)
        {
            if (targetAtomJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.topEnumValFlag)
            {
                if (targetAtomJSMultiEnum.mainVal == oldName)
                {
                    ResetTargetAtomChoices(GetTargetAtomNames());
                    targetAtomJSMultiEnum.mainVal = newName;
                }
            }
        }

        public override void RefreshUI()
        {
            base.RefreshUI();           

            //Merge Load Toggle
            if (resourceType==ResourceType.scene || (resourceCategory == ResourceCategory.preset && targetAtomJSMultiEnum.valType== JSONStorableMultiEnumStringChooser.mainValFlag))
            {
                var mergeToggle = CreateToggle(parentBoxGO, mergePresetJSB, 80f, 20f, "Merge Load", 20, edgeWidth, -edgeWidth,AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(mergeToggle, "Enable Merge Load option when loading the resource.");
            }

            // Open Plugins UI Toggle
            if (resourceType == ResourceType.plugins)
            {
                var openPluginUIToggle = CreateToggle(parentBoxGO, openPluginUIJSB, 100f, 20f, "Open Plugin UI", 20, edgeWidth, -edgeWidth, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(openPluginUIToggle, "The Plugin's UI will be opened in the main VAM UI when the plugin is loaded into the target Atom.");
            }

            // Target Atom
            if (resourceCategory == ResourceCategory.preset || resourceType == ResourceType.plugins)
            {
                targetAtomJSMultiEnum.setTopEnumCallbackFunction = TargetAtomTopEnumCallBack;
                targetAtomJSMultiEnum.setMainValCallbackFunction = TargetAtomMainEnumCallBack;

                targetAtomPopup = CreateScrollablePopup(parentBoxGO, targetAtomJSMultiEnum, 160f, 30f, "Target Atom", 20,edgeWidth, -edgeWidth-20f,300f, true, true, AnchorPresets.topLeft);
                targetAtomPopup.popup.onOpenPopupHandlers += RefreshTargetAtomPopupCallback;
                parentBrowser.ApplyHelp(targetAtomPopup, "Select a target atom from the current scene that the primary action for the selected resource will be performed on.");
                RefreshTargetAtomValue();
            }

            // Main Load Button
            string resourceTypeName = parentBrowser.browserFiltersUI.resourceTypeName;
            if (resourceCategory == ResourceCategory.preset)
            {
                resourceTypeName = resourceTypeName.Substring(7);
                mainActionButton = CreateUIButton(parentBoxGO, 100f, 30f, "Load " + resourceTypeName + " Preset", 20, 165f+ edgeWidth, -edgeWidth-20f, AnchorPresets.topLeft);
            }
            else if (resourceType == ResourceType.scene)
            {
                mainActionButton = CreateUIButton(parentBoxGO, 100f, 30f, "Load Scene", 20, edgeWidth, -edgeWidth-20f, AnchorPresets.topLeft);
            }
            else if (resourceType == ResourceType.plugins)
            {
                mainActionButton = CreateUIButton(parentBoxGO, 100f, 30f, "Load Plugin", 20, 165f + edgeWidth, -edgeWidth-20f, AnchorPresets.topLeft);
            }            
            if (mainActionButton != null)
            {
                mainActionButton.buttonColor = new Color(0, 0.6f, 0);
                mainActionButton.textColor = Color.white;
                mainActionButton.button.onClick.AddListener(delegate
                {
                    if (primaryRVGE != null) Globals.mvrScript.StartCoroutine(primaryRVGE.Activate(this));
                    deleteConfirmationActive = false;
                    RefreshUI();
                });
                PrimaryActionPointerEnterExitHelp primaryActionContextHelp = mainActionButton.button.gameObject.AddComponent<PrimaryActionPointerEnterExitHelp>();
                primaryActionContextHelp.Init(parentBrowser.helpTextJSS, parentBrowser.resourceSelectionsUI.GetResourceTypeSelections(resourceType), this);
                if (resourceType!=ResourceType.scene || !parentBrowser.isVREnabled)
                {
                    var closeBrowserUIonLoad = CreateToggle(parentBoxGO, Settings.resourceSettings[resourceType].closeBrowserUIonResourceLoadJSB, 140f, 20f, "Close UI on Load", 20, 0, 0, AnchorPresets.topRight);
                    parentBrowser.ApplyHelp(closeBrowserUIonLoad, "Close the BrowserAssist window on Resource Load");
                }
                if (isAtomTargetable)
                {
                    var selectAtomOnLoad = CreateToggle(parentBoxGO, Settings.selectAtomOnLoadJSB, 140f, 20f, "VAM Select Atom on Load", 20, 0, -20f, AnchorPresets.topRight);
                    parentBrowser.ApplyHelp(selectAtomOnLoad, "VAM Select the Target atom on Resource Load");

                }


            }

            // Random Load buttons
            if (PC.patreonContentEnabled)
            {
                float randomLoadButtonXOffset = 105f + edgeWidth;
                if (resourceCategory == ResourceCategory.preset || resourceType == ResourceType.plugins)
                {
                    randomLoadButtonXOffset = randomLoadButtonXOffset + 165f;
                }
                var loadRandomButton = CreateUIButton(parentBoxGO, 100f, 30f, "Load Random from current Filter results", 20, randomLoadButtonXOffset, -edgeWidth - 20f, AnchorPresets.topLeft);
                loadRandomButton.buttonColor = new Color(0, 0.6f, 0);
                loadRandomButton.textColor = Color.white;
                loadRandomButton.button.onClick.AddListener(delegate
                {
                    PC.LoadRandomFromFilterResults(parentBrowser, this);
                    deleteConfirmationActive = false;
                    RefreshUI();
                });
                parentBrowser.ApplyHelp(loadRandomButton, "Load a random resource from all the resources in the browser results");
                if (parentBrowser.resourceSelectionsUI.SelectionCount(resourceType) > 1 && parentBrowser.browserFiltersUI.selectModeJSEnum.val == BrowserSelectMode.multi)
                {
                    var loadRandomSelectionButton = CreateUIButton(parentBoxGO, 100f, 30f, "Load Random from current Selections", 20, randomLoadButtonXOffset + 105f, -edgeWidth - 20f, AnchorPresets.topLeft);
                    loadRandomSelectionButton.buttonColor = new Color(0, 0.6f, 0);
                    loadRandomSelectionButton.textColor = Color.white;
                    loadRandomSelectionButton.button.onClick.AddListener(delegate
                    {
                        PC.LoadRandomFromSelections(parentBrowser, resourceType, this);
                        deleteConfirmationActive = false;
                        RefreshUI();
                    });
                    parentBrowser.ApplyHelp(loadRandomSelectionButton, "Load a random resource from the resources that are currently selected");
                }
            }
            

            // App Preset Suppressions Suppress & Other preset options
            if (resourceType == ResourceType.presetAppearance )
            {
                var suppressClothingLoadToggle = CreateToggle(parentBoxGO, suppressClothingLoadJSB, 140f, 20f, "Suppress Clothing Load", 20, edgeWidth, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(suppressClothingLoadToggle, "Any clothing items in the Appearance Preset will be excluded and any clothing on the target atom will reamin");

                if (PC.patreonContentEnabled)
                {
                    var suppressScaleChangeToggle = CreateToggle(parentBoxGO, suppressScaleChangeJSB, 140f, 20f, "Suppress Scale Change", 20, edgeWidth + 140f, -edgeWidth - 50, AnchorPresets.topLeft);
                    parentBrowser.ApplyHelp(suppressScaleChangeToggle, "The scale of the target atom will remain unaffected by the Appearance Preset Load.");
                }
            }
            else if(resourceType == ResourceType.presetMorphs)
            {
                var morphPresetIncludeAppToggle = CreateToggle(parentBoxGO, morphPresetIncludeAppJSB, 120f, 20f, "Include Appearance", 20, edgeWidth, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(morphPresetIncludeAppToggle, "Appearance Morphs will be included in the Preset Load.");
                var morphPresetIncludePhysical = CreateToggle(parentBoxGO, morphPresetIncludePhysicalJSB, 140f, 20f, "Include Physical (Pose)", 20, edgeWidth + 120f, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(morphPresetIncludePhysical, "Pose Morphs will be included in the Preset Load.");
            }
            else if (resourceType == ResourceType.presetGeneral)
            {
                var generalPresetIncludePhysicalToggle = CreateToggle(parentBoxGO, generalPresetIncludePhysicalJSB, 120f, 20f, "Include Physical", 20, edgeWidth, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(generalPresetIncludePhysicalToggle, "Physical Parameters will be included in the General Preset Load.");
                var generalPresetIncludePoseToggle = CreateToggle(parentBoxGO, generalPresetIncludePoseJSB, 100f, 20f, "Include Pose", 20, edgeWidth+120f, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(generalPresetIncludePhysicalToggle, "Pose Parameters will be included in the General Preset Load.");
                var generalPresetIncludeAppToggle = CreateToggle(parentBoxGO, generalPresetIncludeAppJSB, 125f, 20f, "Include Appearance", 20, edgeWidth+220f, -edgeWidth - 50, AnchorPresets.topLeft);
                parentBrowser.ApplyHelp(generalPresetIncludePhysicalToggle, "Appearance Parameters will be included in the General Preset Load.");
            }

            //Delete buttons
            if (primaryRVGE != null && primaryRVGE.isDeletable)
            {
                if (!deleteConfirmationActive)
                {
                    string label = "Delete\nFile...";
                    string helpText = "Delete '" + primaryRVGE.resourceFullPathReference + "'";
                    if (!primaryRVGE.localFileResource)
                    {
                        label = "Delete\nVAR...";
                        helpText = "Delete the VAR Package containing '" + primaryRVGE.resourceFullPathReference + "'.\nWARNING: This will permanently delete all resources from the package.";
                    }
                    var deleteRVGEButton = CreateUIButton(parentBoxGO, 70f, 30f, label, 22, 0, edgeWidth, AnchorPresets.bottomRight);
                    deleteRVGEButton.buttonColor = Color.red;
                    deleteRVGEButton.buttonText.color = Color.white;

                    deleteRVGEButton.button.onClick.AddListener(delegate
                    {
                        deleteConfirmationActive = true;
                        RefreshUI();
                    });

                    parentBrowser.ApplyHelp(deleteRVGEButton, helpText,true);
                }
                else
                {
                    string confirmHelpText = "Confirm deletion of '" + primaryRVGE.resourceFullPathReference + "'";
                    string cancelHelpText = "Cancel deletion of '" + primaryRVGE.resourceFullPathReference + "'";

                    if (!primaryRVGE.localFileResource)
                    {
                        confirmHelpText = "Confirm deletion of VAR containing '" + primaryRVGE.resourceFullPathReference + "'.\nWARNING: This will delete all resources in the containing package.";
                    }
                    var confirmDeleteRVGEButton = CreateUIButton(parentBoxGO, 70f, 30f, "Confirm Deletion", 22, -75f- edgeWidth, edgeWidth, AnchorPresets.bottomRight);
                    confirmDeleteRVGEButton.buttonColor = Color.red;
                    confirmDeleteRVGEButton.buttonText.color = Color.white;
                    confirmDeleteRVGEButton.button.onClick.AddListener(delegate
                    {
                        DeletePrimaryRVGE();
                        deleteConfirmationActive = false;
                        RefreshUI();
                    });
                    parentBrowser.ApplyHelp(confirmDeleteRVGEButton, confirmHelpText,true);

                    var cancelDeleteRVGEButton = CreateUIButton(parentBoxGO, 70f, 30f, "Cancel Deletion", 22, -edgeWidth, edgeWidth, AnchorPresets.bottomRight);
                    cancelDeleteRVGEButton.buttonColor = new Color(0f,0.6f,0f);
                    cancelDeleteRVGEButton.buttonText.color = Color.white;
                    cancelDeleteRVGEButton.button.onClick.AddListener(delegate
                    {
                        deleteConfirmationActive = false;
                        RefreshUI();
                    });
                    parentBrowser.ApplyHelp(cancelDeleteRVGEButton, cancelHelpText);
                }
                
            }
        }
        private void DeletePrimaryRVGE()
        {
            bool lastPackagedVersion = false;
            if (primaryRVGE.localFileResource)
            {
                FileManagerSecure.DeleteFile(Const.SymLinksFolder + "\\" + primaryRVGE.resourceFullPathReference);
                ResourceManifest.PurgeLocalRVGE(primaryRVGE as LocalRVGE);
                
            }
            else
            {
                PackagedRVGE packagedRVGE = primaryRVGE as PackagedRVGE;
                string varFilePath = VARPackageManifest.GetVARPath(packagedRVGE, packagedRVGE.highestVARVersion);

                if (FileManagerSecure.FileExists(Const.SymLinksFolder + "\\" + varFilePath)) FileManagerSecure.DeleteFile(Const.SymLinksFolder + "\\" + varFilePath);

                VARPackageManifest.PurgeVARFile(packagedRVGE.creatorName, packagedRVGE.packageName, packagedRVGE.highestVARVersion);
                if (!VARPackageManifest.varPackageVersionGroups.ContainsKey(VARPackageManifest.GetVPVGEKey(packagedRVGE.creatorName, packagedRVGE.packageName))) lastPackagedVersion =true;
            }
            if (primaryRVGE.localFileResource || lastPackagedVersion) parentBrowser.resourceSelectionsUI.DeSelectRVGE(primaryRVGE);
            parentBrowser.RefreshFilterResults();
        }

        public void RefreshTargetAtomValue()
        {
            List<string> targetAtomNames = GetTargetAtomNames();
            ResetTargetAtomChoices(targetAtomNames);
 
            if (targetAtomJSMultiEnum.valType != JSONStorableMultiEnumStringChooser.topEnumValFlag)
            {
                if (targetAtomNames.Count == 0)
                {
                    if (resourceType == ResourceType.plugins) targetAtomJSMultiEnum.valTopEnum = PluginTargetTypes.sessionPlugins;
                    else targetAtomJSMultiEnum.valTopEnum = CreateNewAtom.newAtom;
                }
                else
                {
                    
                    if (!targetAtomNames.Contains(targetAtomJSMultiEnum.mainVal))
                    {
                        SetFirstPreferredAtom(targetAtomNames);
                    }
                }
            }
            
        }

        private void SetFirstPreferredAtom(List<string> targetAtomNames)
        {
            var maleAtomNames = AtomUtils.GetPersonAtomNames(false, true);
            var femaleAtomNames = AtomUtils.GetPersonAtomNames(true, false);

            if (Settings.defaultTargetPersonTypeJSE.val == PersonGenderTypes.firstFemale && femaleAtomNames.Count > 0) targetAtomJSMultiEnum.mainValNoCallback = femaleAtomNames[0];
            else if (Settings.defaultTargetPersonTypeJSE.val == PersonGenderTypes.firstMale && maleAtomNames.Count > 0)
                targetAtomJSMultiEnum.mainValNoCallback = maleAtomNames[0];
            else targetAtomJSMultiEnum.mainValNoCallback = targetAtomNames[0];
        }
        public void ResetTargetAtomValue()
        {
            var personAtoms = AtomUtils.GetAtomNames(false, false, false, true, false);
            List<string> targetAtomNames = GetTargetAtomNames();
            ResetTargetAtomChoices(targetAtomNames);

            if (resourceType == ResourceType.plugins)
            {
                if (primaryRVGE.IsTagged(TagManifest.GetTag(resourceType, "session")))
                {
                    targetAtomJSMultiEnum.valTopEnum = PluginTargetTypes.sessionPlugins;
                    return;
                }
                else if (primaryRVGE.IsTagged(TagManifest.GetTag(ResourceType.plugins, "scene")))
                {
                    targetAtomJSMultiEnum.valTopEnum = PluginTargetTypes.scenePlugins;
                    return;
                }
                else if (personAtoms.Count > 0)
                {
                    foreach (var person in personAtoms)
                    {
                        if (targetAtomNames.Contains(person))
                        {
                            targetAtomJSMultiEnum.mainVal = person;
                            return;
                        }
                    }
                }
            }
            if (resourceCategory == ResourceCategory.preset)
            {
                if (primaryRVGE.IsTagged(TagManifest.GetTag(resourceType, "female")))
                {
                    if (targetAtomJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag && AtomUtils.IsFemale(SuperController.singleton.GetAtomByUid(targetAtomJSMultiEnum.mainVal))) return;
                    foreach (var person in personAtoms)
                    {
                        if (AtomUtils.IsFemale(SuperController.singleton.GetAtomByUid(person)))
                        {
                            targetAtomJSMultiEnum.mainVal = person;
                            return;
                        }
                    }
                }
                if (primaryRVGE.IsTagged(TagManifest.GetTag(resourceType, "male")))
                {
                    if (targetAtomJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.mainValFlag && AtomUtils.IsMale(SuperController.singleton.GetAtomByUid(targetAtomJSMultiEnum.mainVal))) return;
                    foreach (var person in personAtoms)
                    {
                        if (AtomUtils.IsMale(SuperController.singleton.GetAtomByUid(person)))
                        {
                            targetAtomJSMultiEnum.mainVal = person;
                            return;
                        }
                    }
                }
            }
            

        }


        private List<string> GetTargetAtomNames()
        {
            List<string> targetAtomNames = new List<string>();
            

            if (resoureTargetCategory != ResourceTargetCategory.nonAtomTargetable)
            {
                List<Atom> atoms;

                if (resoureTargetCategory == ResourceTargetCategory.personAtomTargetable) atoms = AtomUtils.GetAtoms(incudeNonPersonAtoms: false);
                else if (resoureTargetCategory == ResourceTargetCategory.otherAtomTargetable) atoms = AtomUtils.GetAtoms(includePersonAtoms: false);
                else atoms = AtomUtils.GetAtoms();

                foreach (Atom atom in atoms) targetAtomNames.Add(atom.name);
            }
            targetAtomNames.Sort();
            return targetAtomNames;
        }

        public void RefreshTargetAtomPopupCallback()
        {
            RefreshTargetAtomPopup();
        }
        public void RefreshTargetAtomPopup(bool forceAtomChoice = false)
        {
            var targetAtomNames = GetTargetAtomNames();
            ResetTargetAtomChoices(targetAtomNames);
            
            if (targetAtomNames.Count > 0 && forceAtomChoice && !targetAtomNames.Contains(targetAtomJSMultiEnum.mainVal))
            {
                SetFirstPreferredAtom(targetAtomNames);
            }
        }

        private void ResetTargetAtomChoices(List<string> atomNames)
        {
            if (resourceType == ResourceType.plugins) targetAtomJSMultiEnum.SetTopEnum(PluginTargetTypes.enumManifestName);
            else targetAtomJSMultiEnum.SetTopEnum(CreateNewAtom.enumManifestName);
            if (atomNames.Count == 0) targetAtomJSMultiEnum.SetMainChoices(null);           
            else targetAtomJSMultiEnum.SetMainChoices(atomNames);
            if (targetAtomJSMultiEnum.popup != null)
            {
                if (atomNames.Count > 10) targetAtomJSMultiEnum.popup.popupPanelHeight = 250f;
                else targetAtomJSMultiEnum.popup.popupPanelHeight = (atomNames.Count + 1) * 25f;
            }
            
        }
    }

    public class ResourceTagsActionUI : ResourceActionUIBase
    {
        private JSONStorableBool hiddenJSB;
        private JSONStorableBool favouriteJSB;
        private JSONStorableBool newJSB;

        private TriStateToggle hideToggle;
        private TriStateToggle favToggle;
        private TriStateToggle newToggle;

        private string tagNameFilter = "";

        private JSONStorableString tagFilterJSS;
        private JSONStorableEnumStringChooser tagCategoryFilterJSEnum;

        private ResourceSelections resourceSelectionsUI { get { return parentBrowser.resourceSelectionsUI; } }

        private BrowserFiltersUI browserFiltersUI { get { return parentBrowser.browserFiltersUI; } }

        private int filtersResourceType { get { return browserFiltersUI.resourceType; } }

        private int filtersSelectionMode { get { return browserFiltersUI.selectModeJSEnum.val; } }

        private GameObject tagSelectionsScrollableBoxGO;
        private GameObject tagSelectionScrollableViewportContentGO;
        private List<GameObject> tagSelectionRows = new List<GameObject>();
        private UIDynamicButton createNewTagButton;
        private UIDynamicButton clearTagFilterButton;

        
        private Dictionary<JSONStorableBool, TriStateToggle> tagJSBtoTriStateToggle = new Dictionary<JSONStorableBool, TriStateToggle>();
        private Dictionary<JSONStorableBool, Tag> tagJSBtoTag = new Dictionary<JSONStorableBool, Tag>();

        public ResourceTagsActionUI(BrowserUI _parentBrowser, GameObject _parentBoxGO) : base(_parentBrowser, _parentBoxGO)
        {
            hiddenJSB = new JSONStorableBool("hidden", false, HiddenCallback);
            favouriteJSB = new JSONStorableBool("favourite", false, FavouriteCallback);
            newJSB = new JSONStorableBool("new", false, NewCallback);
            tagFilterJSS = new JSONStorableString("tagFilter", "");
            tagCategoryFilterJSEnum = new JSONStorableEnumStringChooser("tagCategoryFilter", TagCategory.enumManifestName, TagCategory.all, "Tag Category");
        }

        public override void ClearUI()
        {
            base.ClearUI();
            GameObject.Destroy(tagSelectionsScrollableBoxGO);
        }

        public override void RefreshUI()
        {
            base.RefreshUI();

            ClearTagSelectionRows();

            tagJSBtoTriStateToggle.Clear();
            tagJSBtoTag.Clear();
                
            if (resourceSelectionsUI.SelectionCount(filtersResourceType)>0)
            {
                hideToggle = CreateTriStateToggle(parentBoxGO, hiddenJSB, 80, 20, "Hide", 20, -2, -3, anchorPreset: AnchorPresets.topRight);
                HiddenPointerEnterExitHelp hiddenContextHelp = hideToggle.toggle.gameObject.AddComponent<HiddenPointerEnterExitHelp>();
                hiddenContextHelp.Init(parentBrowser.helpTextJSS, resourceSelectionsUI.GetResourceTypeSelections(browserFiltersUI.resourceType), browserFiltersUI);

                favToggle = CreateTriStateToggle(parentBoxGO, favouriteJSB, 80, 20, "Favorite", 20, -2, -25, anchorPreset: AnchorPresets.topRight);
                FavPointerEnterExitHelp favContextHelp = favToggle.toggle.gameObject.AddComponent<FavPointerEnterExitHelp>();
                favContextHelp.Init(parentBrowser.helpTextJSS, resourceSelectionsUI.GetResourceTypeSelections(browserFiltersUI.resourceType), browserFiltersUI);

                newToggle = CreateTriStateToggle(parentBoxGO, newJSB, 80, 20, "New\\Updated", 20, -2, -45, anchorPreset: AnchorPresets.topRight);
                NewPointerEnterExitHelp newContextHelp = newToggle.toggle.gameObject.AddComponent<NewPointerEnterExitHelp>();
                newContextHelp.Init(parentBrowser.helpTextJSS, resourceSelectionsUI.GetResourceTypeSelections(browserFiltersUI.resourceType), browserFiltersUI);

                if (!Globals.addonPackageFilePrefsFolderAccess || !Globals.customFolderAccess || !Globals.scenesFolderAccess)
                {
                    favToggle.toggle.interactable = false;
                    hideToggle.toggle.interactable = false;

                    parentBrowser.ApplyHelp(hideToggle.toggleUID, "Enable BA Sym Links using .bat file provided in PluginData folder to allow toggling the VAM Hidden state from within Browser Assist.");
                    parentBrowser.ApplyHelp(favToggle.toggleUID, "Enable BA Sym Links using .bat file provided in PluginData folder to allow toggling the VAM Favorite state from within Browser Assist.");
                }

                hideToggle.RefreshToggle(resourceSelectionsUI.IsSelectionHidden(filtersResourceType, filtersSelectionMode));

                favToggle.RefreshToggle(resourceSelectionsUI.IsSelectionFavourite(filtersResourceType, filtersSelectionMode));

                newToggle.RefreshToggle(resourceSelectionsUI.IsSelectionNew(filtersResourceType, filtersSelectionMode));

                var textFieldUID = CreateTextField(parentBoxGO, tagFilterJSS, 150f, 20f, 20, 2, -2, AnchorPresets.topLeft, true, InputField.LineType.SingleLine, "Filter or create new tag...", 0, 0,TagFilterNameCallback);
                parentBrowser.ApplyHelp(textFieldUID, "Enter text to filter the list of Tags that are available for this resource type. Create new Tags, where an existing Tag does not match the entered search text.");

                clearTagFilterButton = CreateUIButton(parentBoxGO, 16f, 16f, "X", 20, 134f, -4, AnchorPresets.topLeft);
                clearTagFilterButton.button.onClick.AddListener(delegate { tagFilterJSS.val=""; RefreshUI(); });
                parentBrowser.ApplyHelp(clearTagFilterButton, "Clear the Tag search field of user entered text.");
                if (tagFilterJSS.val == "") clearTagFilterButton.gameObject.SetActive(false);

                createNewTagButton = CreateUIButton(parentBoxGO, 80f, 20f, "Create Tag", 22, 155, -2, AnchorPresets.topLeft);
                createNewTagButton.buttonColor = Color.green;
                createNewTagButton.textColor = Color.black;
                RefreshNewTagButton();
                createNewTagButton.button.onClick.AddListener(delegate {
                    var newTag = TagManifest.CreateTag(tagNameFilter, TagCategory.userDefined, filtersResourceType);
                    resourceSelectionsUI.SetSelectionTag(filtersResourceType, filtersSelectionMode, newTag, TriState.active);
                    RefreshUI();
                });
                parentBrowser.ApplyHelp(createNewTagButton, "Create a new Tag and assign it to the selected resources.");

//                var tagCategorryFilterUID = CreatePopup(parentBoxGO, tagCategoryFilterJSEnum, 160f, 20f, "Tag Category", 20, tagSelectionBoxWidth - 160f, -2f,AnchorPresets.topLeft,true);

                tagSelectionsScrollableBoxGO = UI.CreateVScrollableContent(parentBoxGO, new Color(0.5f, 0.5f, 0.65f, 0.98f), AnchorPresets.topLeft, tagSelectionBoxWidth, 48f, new Vector2(2f, -24f));
                tagSelectionScrollableViewportContentGO = tagSelectionsScrollableBoxGO.transform.Find("scrollableContentViewportGO/vScrollableContentGO").gameObject;

                RePopulateTagSelectionsBox();
            }
            
        }

        private void RefreshNewTagButton()
        {
            if (tagNameFilter == "hide" || tagNameFilter == "favorite" || tagNameFilter.EndsWith(" ") || tagNameFilter =="" || TagManifest.GetTags(filtersResourceType).Select(x => x.tagName).Contains(tagNameFilter)) createNewTagButton.gameObject.SetActive(false);
            else createNewTagButton.gameObject.SetActive(true);
        }

        private void TagFilterNameCallback(string i)
        {
            tagNameFilter = TagManifest.NormalizeTagName(i);
            tagFilterJSS.val = tagNameFilter;
            tagFilterJSS.inputField.text = tagNameFilter;
            if (tagFilterJSS.val == "") clearTagFilterButton.gameObject.SetActive(false);
            else clearTagFilterButton.gameObject.SetActive(true);

            RefreshNewTagButton();
            RePopulateTagSelectionsBox();

        }
        private void ClearTagSelectionRows()
        {
            foreach (var rowGO in tagSelectionRows) GameObject.Destroy(rowGO);
            tagSelectionRows.Clear();
        }

        const float tagSelectionBoxWidth = 550f;
        private GameObject CreateSelectTagRow()
        {
            GameObject currentRowGO = UI.AddChildGOImage(tagSelectionScrollableViewportContentGO, new Color(0f, 0.8f, 0f, 0f), AnchorPresets.hStretchTop, tagSelectionBoxWidth, 20f, Vector2.zero);
            tagSelectionRows.Add(currentRowGO);

            return currentRowGO;
        }

        private float GetTagButtonWidth(Tag tag)
        {
            return Mathf.Clamp((tag.tagName.Length * 5)+25, 40f, 150f);

        }

        public void RePopulateTagSelectionsBox()
        {
            foreach (var toggleKVP in tagJSBtoTriStateToggle)
            {

                GameObject.Destroy(toggleKVP.Value.toggleUID.gameObject);
                uiElements.Remove(toggleKVP.Value.toggleUID);
            }

            ClearTagSelectionRows();

            tagJSBtoTriStateToggle.Clear();
            tagJSBtoTag.Clear();

            float currentRowTagsWidth = 0f;

            List<Tag> availableTags = TagManifest.GetTags(filtersResourceType).Where(t => t.tagName.StartsWith(tagNameFilter)).ToList();

            if (availableTags.Count == 0) return;

            GameObject currentRowGO = CreateSelectTagRow();
            foreach (var tag in availableTags)
            {
                float nextButtonWidth = GetTagButtonWidth(tag);

                float maxWidthBroder = 0f;
                if (Globals.isVREnabled) maxWidthBroder = 25f;

                if (currentRowTagsWidth + nextButtonWidth  > (tagSelectionBoxWidth - maxWidthBroder))
                {
                    currentRowGO = CreateSelectTagRow();
                    currentRowTagsWidth = 0f;
                }
                var tagJSB = new JSONStorableBool(tag.tagName, false, TagCallback);
                var tagToggle = CreateTriStateToggle(currentRowGO, tagJSB, nextButtonWidth, 20f, tag.tagName, 20, currentRowTagsWidth, 0, AnchorPresets.middleLeft);
                

                tagToggle.RefreshToggle(resourceSelectionsUI.IsSelectionTagged(filtersResourceType, filtersSelectionMode, tag));
                if (tag.tagCategory == TagCategory.combi)
                {
                    tagToggle.toggle.interactable = false;
                    tagToggle.toggleUID.textColor = Color.white;
                }

                tagJSBtoTag.Add(tagJSB, tag);
                tagJSBtoTriStateToggle.Add(tagJSB, tagToggle);

                tagToggle.backgroundColor = TagCategory.GetColor(tag.tagCategory);

               TagPointerEnterExitHelp contextHelp = tagToggle.toggle.gameObject.AddComponent<TagPointerEnterExitHelp>();
                contextHelp.Init(tag, parentBrowser.helpTextJSS, resourceSelectionsUI.GetResourceTypeSelections(browserFiltersUI.resourceType),browserFiltersUI);

                currentRowTagsWidth += nextButtonWidth;

                currentRowTagsWidth += 3f;
            }
        }

        public void TagCallback(JSONStorableBool valueJSB)
        {
            int state = resourceSelectionsUI.IsSelectionTagged(filtersResourceType, filtersSelectionMode, tagJSBtoTag[valueJSB]);

            int targetTriState = TriState.active;
            if (state == TriState.active) targetTriState = TriState.inactive;

            resourceSelectionsUI.SetSelectionTag(filtersResourceType, filtersSelectionMode, tagJSBtoTag[valueJSB], targetTriState);
            tagJSBtoTriStateToggle[valueJSB].RefreshToggle(targetTriState);
            RefreshUI();

            if (parentBrowser.browserFiltersUI.tagsFilterJSMultiEnum.valType!=JSONStorableMultiEnumStringChooser.topEnumValFlag || parentBrowser.browserFiltersUI.tagsFilterJSMultiEnum.valTopEnum!=TagGroups.anyTags) parentBrowser.RefreshFilterResults();
        }

        public void HiddenCallback(bool value)
        {
            int state = resourceSelectionsUI.IsSelectionHidden(filtersResourceType, filtersSelectionMode);

            if (state == TriState.active)
            {
                resourceSelectionsUI.SetSelectionHidden(filtersResourceType, filtersSelectionMode, TriState.inactive);

                hideToggle.RefreshToggle(resourceSelectionsUI.IsSelectionHidden(filtersResourceType, filtersSelectionMode));
            }
            else
            {
                resourceSelectionsUI.SetSelectionHidden(filtersResourceType, filtersSelectionMode, TriState.active);
                hideToggle.RefreshToggle(resourceSelectionsUI.IsSelectionHidden(filtersResourceType, filtersSelectionMode));
            }

            RefreshUI();
            if (!parentBrowser.browserFiltersUI.showHiddenJSB.val)parentBrowser.RefreshFilterResults();


        }

        public void FavouriteCallback(bool value)
        {
            int state = resourceSelectionsUI.IsSelectionFavourite(filtersResourceType, filtersSelectionMode);

            if (state == TriState.active)
            {
                resourceSelectionsUI.SetSelectionFavourite(filtersResourceType, filtersSelectionMode, TriState.inactive);
                favToggle.RefreshToggle(TriState.inactive);
            }
            else
            {
                resourceSelectionsUI.SetSelectionFavourite(filtersResourceType, filtersSelectionMode, TriState.active);
                favToggle.RefreshToggle(TriState.active);
            }

            RefreshUI();
            if (parentBrowser.browserFiltersUI.onlyFavouritesJSB.val) parentBrowser.RefreshFilterResults();
        }

        public void NewCallback(bool value)
        {
            int state = resourceSelectionsUI.IsSelectionNew(filtersResourceType, filtersSelectionMode);

            if (state == TriState.active)
            {
                resourceSelectionsUI.SetSelectionNew(filtersResourceType, filtersSelectionMode, TriState.inactive);
                newToggle.RefreshToggle(TriState.inactive);
            }
            else
            {
                resourceSelectionsUI.SetSelectionNew(filtersResourceType, filtersSelectionMode, TriState.active);
                newToggle.RefreshToggle(TriState.active);
            }
            RefreshUI();
            if (parentBrowser.browserFiltersUI.onlyNewJSB.val) parentBrowser.RefreshFilterResults();
        }

    }

    public class ResourcePositionActionUI : ResourceActionUIBase
    {

        private bool isNewAtomCreation { get { return resourceActionTab.targetAtomJSMultiEnum.valType == JSONStorableMultiEnumStringChooser.topEnumValFlag; } }

        private int resourceType { get { return parentBrowser.browserFiltersUI.resourceType; } }
        private int resourceCategory { get { return ResourceType.GetResourceCategory(resourceType); } }

        private ResourcePrimaryActionUI resourceActionTab;

        private GameObject rotationLabelGO;
        private GameObject positionLabelGO;

        public static AtomPositionRotationSync existingAtomPosition ;
        public static RelativeAtomPositionRotation newAtomPosition ;
        public ResourcePositionActionUI(BrowserUI _parentBrowser, GameObject _parentBoxGO, ResourcePrimaryActionUI _resourceActionTab) : base(_parentBrowser, _parentBoxGO)
        {
            resourceActionTab = _resourceActionTab;

            newAtomPosition.Init(RelativeAbsoluteChangeCallback, RelativeToAtomCallback, RelativeToAtomCallback,true);

            
            if (Globals.isVREnabled) newAtomPosition.CopyFrom(Settings.vrDefaultNewAtomPosRot);
            else newAtomPosition.CopyFrom(Settings.desktopDefaultNewAtomPosRot);
        }

        public override void AtomNameUpdated(string oldName, string newName)
        {
            if (newAtomPosition!=null) newAtomPosition.AtomNameUpdated(oldName, newName);  
        }

        public override void OnAtomRemoved(Atom atom)
        {
            if (newAtomPosition != null) newAtomPosition.OnAtomRemoved(atom);
        }

        private void RelativeAbsoluteChangeCallback(int val)
        {
            RefreshUI();
        }

        private void RelativeToAtomCallback(int val)
        {
            RefreshUI();
        }

        private void RelativeToAtomCallback(string val)
        {
            RefreshUI();
        }

        public override void ClearUI()
        {
            base.ClearUI();
            
            newAtomPosition.SetGuideAtomVisible(false);
            if (rotationLabelGO != null) GameObject.Destroy(rotationLabelGO);
            if (positionLabelGO != null) GameObject.Destroy(positionLabelGO);

            if (posOptionsPopupGO!=null) GameObject.Destroy(posOptionsPopupGO);
            if (rotOptionsPopupGO!=null) GameObject.Destroy(rotOptionsPopupGO);
            currentPositionRotationAxis = -1;
        }

        public override void RefreshUI()
        {
            base.RefreshUI();
            // Target Atom
            if (resourceCategory == ResourceCategory.preset || resourceType == ResourceType.plugins)
            {
                resourceActionTab.targetAtomJSMultiEnum.setTopEnumCallbackFunction = TargetAtomTopEnumCallBack;
                resourceActionTab.targetAtomJSMultiEnum.setMainValCallbackFunction = TargetAtomMainEnumCallBack;

                var targetAtomPopup = CreateScrollablePopup(parentBoxGO, resourceActionTab.targetAtomJSMultiEnum, 160f, 25f, "Target Atom", 20, edgeWidth, -edgeWidth, 500f, true, true, AnchorPresets.topLeft);
                targetAtomPopup.popup.onOpenPopupHandlers += resourceActionTab.RefreshTargetAtomPopupCallback;
                parentBrowser.ApplyHelp(targetAtomPopup, "Select a target atom from the current scene to reposition. Select '<New Atom>' to specify the position of the next new atom to be created.");
                resourceActionTab.RefreshTargetAtomValue();

                AtomPositionRotationSync currentAtomPosition = existingAtomPosition;

                float relAbsLabelProportion = 0.4f;
                if (isNewAtomCreation )
                {
                    newAtomPosition.SetGuideAtomVisible(true);
                    currentAtomPosition = newAtomPosition;
                    if (newAtomPosition.isRelativePosition || newAtomPosition.isRelativeRotation)
                    {
                        var relativeAtomPopup = CreateScrollablePopup(parentBoxGO, newAtomPosition.relativeAtomNameJSME, 160f, 25f, "Relative to Atom",18,edgeWidth,-edgeWidth-25f,500f,true,true,AnchorPresets.topLeft,20);
                        relativeAtomPopup.popup.onOpenPopupHandlers += RefreshRelativeAtomPopup;
                        parentBrowser.ApplyHelp(relativeAtomPopup, "New atoms will be created relative to this atom (or VR/Dekstop control point). Only the position and rotation elements marked as REL will be relative to this atom. Atom creation can also be relative to the VR Headset, Controllers or the main desktop view (ie. in front of you).");


                        if (newAtomPosition.relativeAtomNameJSME.valType == JSONStorableMultiEnumStringChooser.mainValFlag)
                        {
                            var relativeAtom = SuperController.singleton.GetAtomByUid(newAtomPosition.relativeAtomNameJSME.mainVal);
                            if (relativeAtom !=null && relativeAtom.type == "Person")
                            {
                                var relativeNodePopup = CreateScrollablePopup(parentBoxGO, newAtomPosition.relativeNodeNameJSSC, 160f, 25f, "Relative to Node", 18, edgeWidth, -edgeWidth - 50f, 500f, true, true, AnchorPresets.topLeft, 20);
                                relativeNodePopup.popup.onOpenPopupHandlers += RefreshRelativeNodePopup;
                                parentBrowser.ApplyHelp(relativeNodePopup, "New atoms will be created relative to this atom node. Only the position and rotation elements marked as REL will be relative to this contoller.");
                            }
                            
                        }
                        
                    }
                    
                }
                else
                {
                    currentAtomPosition.SetAtomName(resourceActionTab.targetAtomJSMultiEnum.mainVal);
                    relAbsLabelProportion = 1f;
                    if (currentAtomPosition.atomCollisionEnabledJSB != null)
                    {
                        var collisionToggle = CreateToggle(parentBoxGO, currentAtomPosition.atomCollisionEnabledJSB, 160f, 20f, "Collision Enabled", 20, edgeWidth, -edgeWidth - 25f, AnchorPresets.topLeft);
                        parentBrowser.ApplyHelp(collisionToggle, "Collision for the target atom can be toggled on or off to facilitate easier re-positioning.");
                    }
                    
                }

                var posXRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.posXRelAbsJSE, 60, 20, "X", 24, edgeWidth + 165f, -edgeWidth - 10, AnchorPresets.topLeft, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(posXRelAbsPopup, "Choose whether the X axis position for a new Atom is specified relative (Rel) to another position or if it's an absolute (Abs) Global position in World Space.");

                var posYRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.posYRelAbsJSE, 60, 20, "Y", 24, edgeWidth + 165f, -edgeWidth - 10f - 20f, AnchorPresets.topLeft, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(posYRelAbsPopup, "Choose whether the Y axis position (height) for a new Atom is specified relative (Rel) to another position or if it's an absolute (Abs) Global position in World Space.");

                var posZRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.posZRelAbsJSE, 60, 20, "Z", 24, edgeWidth + 165f, -edgeWidth - 10f - 40f, AnchorPresets.topLeft, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(posZRelAbsPopup, "Choose whether the Z axis position for a new Atom is specified relative (Rel) to another position or if it's an absolute (Abs) Global position in World Space.");

                var rotXRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.rotXRelAbsJSE, 60, 20, "X", 24, -160f - edgeWidth, -edgeWidth - 10, AnchorPresets.topRight, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(rotXRelAbsPopup, "Choose whether the X axis rotation (pitch) for a new Atom is specified relative (Rel) to another rotation or if it's an absolute (Abs) Global rotation in World Space.");

                var rotYRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.rotYRelAbsJSE, 60, 20, "Y", 24, -160f - edgeWidth, -edgeWidth - 10f - 20f, AnchorPresets.topRight, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(rotYRelAbsPopup, "Choose whether the Y axis rotation (yaw) for a new Atom is specified relative (Rel) to another rotation or if it's an absolute (Abs) Global rotation in World Space.");

                var rotZRelAbsPopup = CreatePopup(parentBoxGO, newAtomPosition.rotZRelAbsJSE, 60, 20, "Z", 24, -160f - edgeWidth, -edgeWidth - 10f - 40f, AnchorPresets.topRight, true, relAbsLabelProportion);
                parentBrowser.ApplyHelp(rotZRelAbsPopup, "Choose whether the Z axis rotation (roll) for a new Atom is specified relative (Rel) to another rotation or if it's an absolute (Abs) Global rotation in World Space.");

                positionLabelGO = UI.CreateBasicTextBox(parentBoxGO, new JSONStorableString("", "POSITION"), 100f, 12f, 20, edgeWidth + 165f, -1f, AnchorPresets.topLeft, true, backgoundColor: BrowserUI.darkBlueGrey);
                rotationLabelGO= UI.CreateBasicTextBox(parentBoxGO, new JSONStorableString("", "ROTATION"), 100f, 12f, 20, -120f, -1f, AnchorPresets.topRight, true, backgoundColor: BrowserUI.darkBlueGrey);

                
                var targetAtomPositionXSlider = CreateSlider(parentBoxGO, currentAtomPosition.positionXJSF, 165f,20f,"Postion X",26,edgeWidth+165f+60f,-edgeWidth-10f,  AnchorPresets.topLeft, 0f, 0f, 45f, true, true);
                targetAtomPositionXSlider.valueFormat = "F3";
                parentBrowser.ApplyHelp(targetAtomPositionXSlider, "Adjust the X axis position for the target atom.");
                var xPosOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26, edgeWidth + 165f + 60f + 165f, -edgeWidth - 10f, AnchorPresets.topLeft);
                xPosOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.xPosition);
                });
                parentBrowser.ApplyHelp(xPosOptionsButton, "Toggle options menu for the X Position axis slider");

                var targetAtomPositionYSlider = CreateSlider(parentBoxGO, currentAtomPosition.positionYJSF, 165f, 20f, "Postion Y", 26, edgeWidth + 165f + 60f, -edgeWidth - 10f - 20f, AnchorPresets.topLeft, 0f, 0f, 45f, true, true);
                targetAtomPositionYSlider.valueFormat = "F3";
                parentBrowser.ApplyHelp(targetAtomPositionYSlider, "Adjust the Y axis position for the target atom.");
                var yPosOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26, edgeWidth + 165f + 60f + 165f, -edgeWidth - 10f-20f, AnchorPresets.topLeft);
                yPosOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.yPosition);
                });
                parentBrowser.ApplyHelp(yPosOptionsButton, "Toggle options menu for the Y Position axis slider");


                var targetAtomPositionZSlider = CreateSlider(parentBoxGO, currentAtomPosition.positionZJSF, 165f, 20f, "Postion Z", 26, edgeWidth + 165f + 60f, -edgeWidth - 10f - 40f, AnchorPresets.topLeft, 0f, 0f, 45f, true, true);
                targetAtomPositionZSlider.valueFormat = "F3";
                parentBrowser.ApplyHelp(targetAtomPositionZSlider, "Adjust the Z axis position for the target atom.");
                var zPosOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26, edgeWidth + 165f + 60f + 165f, -edgeWidth - 10f - 40f, AnchorPresets.topLeft);
                zPosOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.zPosition);
                });
                parentBrowser.ApplyHelp(zPosOptionsButton, "Toggle options menu for the Z Position axis slider");

                var targetAtomRotationXSlider = CreateSlider(parentBoxGO, currentAtomPosition.rotationXJSF, 140f, 20f, "Rotation X", 24, -edgeWidth-20f, -edgeWidth-10f, AnchorPresets.topRight, 0f, 0f, 35f,true);
                targetAtomRotationXSlider.valueFormat = "F1";
                parentBrowser.ApplyHelp(targetAtomRotationXSlider, "Adjust the X axis rotation (Pitch) for the target atom.");
                var xRotOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26, -edgeWidth, -edgeWidth - 10f, AnchorPresets.topRight);
                xRotOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.xRotation);
                });
                parentBrowser.ApplyHelp(xRotOptionsButton, "Toggle options menu for the X Rotation axis slider");

                var targetAtomRotationYSlider = CreateSlider(parentBoxGO, currentAtomPosition.rotationYJSF, 140f, 20f, "Rotation Y", 24, -edgeWidth-20f, -edgeWidth - 10f-20f, AnchorPresets.topRight, 0f, 0f, 35f, true);
                targetAtomRotationYSlider.valueFormat = "F1";
                parentBrowser.ApplyHelp(targetAtomRotationYSlider, "Adjust the Y axis rotation (Yaw) for the target atom.");
                var yRotOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26,  -edgeWidth, -edgeWidth - 10f-20f, AnchorPresets.topRight);
                yRotOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.yRotation);
                });
                parentBrowser.ApplyHelp(yRotOptionsButton, "Toggle options menu for the Y Rotation axis slider");

                var targetAtomRotationZSlider = CreateSlider(parentBoxGO, currentAtomPosition.rotationZJSF, 140f, 20f, "Rotation Z", 24, -edgeWidth-20f, -edgeWidth - 10f-40f, AnchorPresets.topRight, 0f, 0f, 35f, true);
                targetAtomRotationZSlider.valueFormat = "F1";
                parentBrowser.ApplyHelp(targetAtomRotationZSlider, "Adjust the Z axis rotation (Roll) for the target atom.");
                var zRotOptionsButton = CreateUIButton(parentBoxGO, 20f, 20f, "^", 26, -edgeWidth, -edgeWidth - 10f-40f, AnchorPresets.topRight);
                zRotOptionsButton.button.onClick.AddListener(delegate
                {
                    ToggleAxisOptionsPopup(PositionRotationAxis.zRotation);
                });
                parentBrowser.ApplyHelp(zRotOptionsButton, "Toggle options menu for the Z Rotation axis slider");


            }
            CreatePostionRotationOptionPopups();
        }

        private int currentPositionRotationAxis = -1;
        private GameObject posOptionsPopupGO = null;
        private GameObject rotOptionsPopupGO = null;

        private void ToggleAxisOptionsPopup(int positionRotationAxis)
        {
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            if (currentPositionRotationAxis== positionRotationAxis)
            {
                
                currentPositionRotationAxis = -1;
            }
            else
            {
                RectTransform rtPos = posOptionsPopupGO.GetComponent<RectTransform>();
                RectTransform rtRot = rotOptionsPopupGO.GetComponent<RectTransform>();
                if (positionRotationAxis == PositionRotationAxis.xPosition)
                {
                    posOptionsPopupGO.SetActive(true);
                    rtPos.anchoredPosition = new Vector2(390f, 60f);
                }
                else if (positionRotationAxis == PositionRotationAxis.yPosition)
                {
                    posOptionsPopupGO.SetActive(true);
                    rtPos.anchoredPosition = new Vector2(390f, 60f-20f);
                }
                else if (positionRotationAxis == PositionRotationAxis.zPosition)
                {
                    posOptionsPopupGO.SetActive(true);
                    rtPos.anchoredPosition = new Vector2(390f, 60f-40f);
                }
                else if (positionRotationAxis == PositionRotationAxis.xRotation)
                {
                    rotOptionsPopupGO.SetActive(true);
                    rtRot.anchoredPosition = new Vector2(570f, 60f);
                }
                else if (positionRotationAxis == PositionRotationAxis.yRotation)
                {
                    rotOptionsPopupGO.SetActive(true);
                    rtRot.anchoredPosition = new Vector2(570f, 60f - 20f);
                }
                else if (positionRotationAxis == PositionRotationAxis.zRotation)
                {
                    rotOptionsPopupGO.SetActive(true);
                    rtRot.anchoredPosition = new Vector2(570f, 60f - 40f);
                }

                currentPositionRotationAxis = positionRotationAxis;
            }
        }

        private void CreatePostionRotationOptionPopups()
        {
            posOptionsPopupGO = UI.AddChildGOImage(parentBoxGO, Color.gray, AnchorPresets.bottomLeft, 75, 105, new Vector2(390, 60));
            rotOptionsPopupGO = UI.AddChildGOImage(parentBoxGO, Color.gray, AnchorPresets.bottomLeft, 75, 45, new Vector2(590, 60));

            var optionButton = UI.CreateUIButton(posOptionsPopupGO, 70, 20, "Zero", 20, 0, -2.5f, AnchorPresets.topMiddle,true);
            optionButton.button.onClick.AddListener(delegate
            {
                ZeroAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Set the current axis slider to Zero value");

            optionButton = UI.CreateUIButton(posOptionsPopupGO, 70, 20, "Default", 20, 0, -20 - 2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                DefaultAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Reset the current axis slider to it's default values");

            optionButton = UI.CreateUIButton(posOptionsPopupGO, 70, 20, "Range x0.1", 20, 0, -40 - 2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                DecreaseRangeAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Decrease the range of the axis slider to 10% of it's current minimum to maximum range");

            optionButton = UI.CreateUIButton(posOptionsPopupGO, 70, 20, "Range x10", 20, 0, -60 - 2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                IncreaseRangeAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Increase the min to max range of the axis slider by 10 times.");

            optionButton = UI.CreateUIButton(posOptionsPopupGO, 70, 20, "Re-centre Range", 20, 0, -80 - 2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                ResetRangeAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Reset the slider min and max values so that the current value is in the centre of the slider");

            optionButton = UI.CreateUIButton(rotOptionsPopupGO, 70, 20, "Zero", 20, 0, -2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                ZeroAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Set the current axis slider to Zero value");

            optionButton = UI.CreateUIButton(rotOptionsPopupGO, 70, 20, "Default", 20, 0, -20 - 2.5f, AnchorPresets.topMiddle, true);
            optionButton.button.onClick.AddListener(delegate
            {
                DefaultAxisOptionsSelected();
            });
            parentBrowser.ApplyHelp(optionButton, "Reset the current axis slider to it's default values");

            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
        }
        private void ZeroAxisOptionsSelected()
        {
            if (currentPositionRotationAxis != -1)
            {
                var jsf = GetSelectedJSF();
                jsf.val = 0f;
                
            }
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            currentPositionRotationAxis = -1;
        }
        private void DefaultAxisOptionsSelected()
        {
            if (currentPositionRotationAxis != -1)
            {
                var jsf = GetSelectedJSF();
                jsf.val = GetDefaultSelectedJSF().val;

                if (isNewAtomCreation)
                {
                    var jse = GetSelectedRelAbsEnum();
                    jse.val = GetDefaultSelectedRelAbsJSEnum().val;
                }

            }
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            currentPositionRotationAxis = -1;
        }
        private void DecreaseRangeAxisOptionsSelected()
        {
            if (currentPositionRotationAxis != -1)
            {
                var jsf = GetSelectedJSF();
                float newMin = jsf.min * 0.1f;
                float newMax = jsf.max * 0.1f;
                if (jsf.val < newMin || jsf.val > newMax)
                {
                    float range = newMax - newMin;
                    jsf.min = jsf.val - (range / 2f);
                    jsf.max = jsf.val + (range / 2f);
                }
                else
                {
                    jsf.min = newMin;
                    jsf.max = newMax;
                }
            }
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            currentPositionRotationAxis = -1;
        }
        private void IncreaseRangeAxisOptionsSelected()
        {
            if (currentPositionRotationAxis != -1)
            {
                var jsf = GetSelectedJSF();
                var range = jsf.max - jsf.min;
                var currentCentre = jsf.min +(range / 2f);

                var newRange = range * 10f;


                jsf.min = currentCentre - (newRange/2f);
                jsf.max = currentCentre + (newRange / 2f);

            }
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            currentPositionRotationAxis = -1;
        }
        private void ResetRangeAxisOptionsSelected()
        {
            if (currentPositionRotationAxis != -1)
            {
                var jsf = GetSelectedJSF();
                float range = jsf.max - jsf.min;
                jsf.min = jsf.val - (range / 2f);
                jsf.max = jsf.val + (range / 2f);

            }
            posOptionsPopupGO.SetActive(false);
            rotOptionsPopupGO.SetActive(false);
            currentPositionRotationAxis = -1;
        }



        private JSONStorableFloat GetSelectedJSF()
        {
            AtomPositionRotationSync currentTargetAtomPosition = existingAtomPosition;
            if (isNewAtomCreation)
            {
                currentTargetAtomPosition = newAtomPosition;
            }
            if (currentPositionRotationAxis == PositionRotationAxis.xPosition) return currentTargetAtomPosition.positionXJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.yPosition) return currentTargetAtomPosition.positionYJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.zPosition) return currentTargetAtomPosition.positionZJSF;

            if (currentPositionRotationAxis == PositionRotationAxis.xRotation) return currentTargetAtomPosition.rotationXJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.yRotation) return currentTargetAtomPosition.rotationYJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.zRotation) return currentTargetAtomPosition.rotationZJSF;

            return null;

        }
        private JSONStorableEnumStringChooser GetSelectedRelAbsEnum()
        {

            if (currentPositionRotationAxis == PositionRotationAxis.xPosition) return newAtomPosition.posXRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.yPosition) return newAtomPosition.posYRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.zPosition) return newAtomPosition.posZRelAbsJSE;

            if (currentPositionRotationAxis == PositionRotationAxis.xRotation) return newAtomPosition.rotXRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.yRotation) return newAtomPosition.rotYRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.zRotation) return newAtomPosition.rotZRelAbsJSE;

            return null;

        }

        private JSONStorableFloat GetDefaultSelectedJSF()
        {
            RelativeAtomPositionRotation defaultNewAtomPositionRotation = Settings.desktopDefaultNewAtomPosRot;
            if (Globals.isVREnabled)
            {
                defaultNewAtomPositionRotation = Settings.vrDefaultNewAtomPosRot;
            }
            if (currentPositionRotationAxis == PositionRotationAxis.xPosition) return defaultNewAtomPositionRotation.positionXJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.yPosition) return defaultNewAtomPositionRotation.positionYJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.zPosition) return defaultNewAtomPositionRotation.positionZJSF;

            if (currentPositionRotationAxis == PositionRotationAxis.xRotation) return defaultNewAtomPositionRotation.rotationXJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.yRotation) return defaultNewAtomPositionRotation.rotationYJSF;
            if (currentPositionRotationAxis == PositionRotationAxis.zRotation) return defaultNewAtomPositionRotation.rotationZJSF;

            return null;

        }

        private JSONStorableEnumStringChooser GetDefaultSelectedRelAbsJSEnum()
        {
            RelativeAtomPositionRotation defaultNewAtomPositionRotation = Settings.desktopDefaultNewAtomPosRot;
            if (Globals.isVREnabled)
            {
                defaultNewAtomPositionRotation = Settings.vrDefaultNewAtomPosRot;
            }
            if (currentPositionRotationAxis == PositionRotationAxis.xPosition) return defaultNewAtomPositionRotation.posXRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.yPosition) return defaultNewAtomPositionRotation.posYRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.zPosition) return defaultNewAtomPositionRotation.posZRelAbsJSE;

            if (currentPositionRotationAxis == PositionRotationAxis.xRotation) return defaultNewAtomPositionRotation.rotXRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.yRotation) return defaultNewAtomPositionRotation.rotYRelAbsJSE;
            if (currentPositionRotationAxis == PositionRotationAxis.zRotation) return defaultNewAtomPositionRotation.rotZRelAbsJSE;

            return null;
        }



        private void RefreshRelativeAtomPopup()
        {
            newAtomPosition.RefreshRelativeAtomChoices();
        }
        private void RefreshRelativeNodePopup()
        {
            newAtomPosition.RefreshRelativeNodeChoices();
        }

        private void TargetAtomTopEnumCallBack(int i)
        {
            newAtomPosition.SetGuideAtomVisible(true);
            RefreshUI();
        }
        private void TargetAtomMainEnumCallBack(string i)
        {
            newAtomPosition.SetGuideAtomVisible(false);
            RefreshUI();
        }

    }


    public class ResourceInfoActionUI : ResourceActionUIBase
    {
        private ResourceSelections resourceSelectionsUI { get { return parentBrowser.resourceSelectionsUI; } }

        protected ResourceVersionGroupEntry primaryRVGE { get { return resourceSelectionsUI.GetFirstSelectedRVGE(filtersResourceType); } }

        private BrowserFiltersUI browserFiltersUI { get { return parentBrowser.browserFiltersUI; } }

        private int filtersResourceType { get { return browserFiltersUI.resourceType; } }

        private int filtersSelectionMode { get { return browserFiltersUI.selectModeJSEnum.val; } }

        private GameObject infoPanelScrollableBoxGO;
        private GameObject infoPaneScrollableViewportContentGO;
        private List<ResourceInfoPanelBase> infoPanelColumns = new List<ResourceInfoPanelBase>();


        public ResourceInfoActionUI(BrowserUI _parentBrowser, GameObject _parentBoxGO) : base(_parentBrowser, _parentBoxGO)
        {

        }

        public override void ClearUI()
        {
            base.ClearUI();
            GameObject.Destroy(infoPanelScrollableBoxGO);
            infoPanelColumns.Clear();
        }

        const float panelBoxHeight = BrowserUI.resourceActionBoxHeight - ResourceActionsUI.resourceActionTabsHeight-4f;

        private float hScrollPosition = 0f;
        public override void RefreshUI()
        {
            base.RefreshUI();

            if (resourceSelectionsUI.SelectionCount(filtersResourceType) > 0 && primaryRVGE!=null)
            {
                infoPanelScrollableBoxGO = UI.CreateHScrollableContent(parentBoxGO, new Color(0.5f, 0.5f, 0.65f, 0f), AnchorPresets.topLeft,636f , panelBoxHeight, new Vector2(2f, -2f));
                infoPaneScrollableViewportContentGO = infoPanelScrollableBoxGO.transform.Find("scrollableContentViewportGO/hScrollableContentGO").gameObject;

                var scrollRect = infoPanelScrollableBoxGO.GetComponent<ScrollRect>();
                scrollRect.horizontalScrollbar.onValueChanged.AddListener(delegate
                {
                    hScrollPosition = scrollRect.horizontalNormalizedPosition;
                });
                

                infoPanelColumns.Add(new ResourceNameFolderInfoPanel(infoPaneScrollableViewportContentGO, panelBoxHeight,parentBrowser));

                if (!primaryRVGE.localFileResource) infoPanelColumns.Add(new PackageInfoPanel(infoPaneScrollableViewportContentGO, panelBoxHeight, parentBrowser));

                infoPanelColumns.Add(new ThumbnailInfoPanel(infoPaneScrollableViewportContentGO, panelBoxHeight, parentBrowser));
                infoPanelColumns.Add(new NotesInfoPanel(infoPaneScrollableViewportContentGO, panelBoxHeight, parentBrowser));

                scrollRect.horizontalNormalizedPosition = hScrollPosition;
            }

        }

    }

    public abstract class ResourceInfoPanelBase
    {
        protected GameObject currentPanelColumnGO;
        protected BrowserUI parentBrowser;
        protected ResourceVersionGroupEntry primaryRVGE { get { return resourceSelectionsUI.GetFirstSelectedRVGE(resourceType); } }

        protected ResourceSelections resourceSelectionsUI { get { return parentBrowser.resourceSelectionsUI; } }

        private int resourceType { get { return parentBrowser.browserFiltersUI.resourceType; } }

        public ResourceInfoPanelBase(GameObject parentViewportGO, float panelBoxHeight, BrowserUI parentBrowser )
        {
            currentPanelColumnGO = UI.AddChildGOImage(parentViewportGO, new Color(0f, 0f, 0f, 0f), AnchorPresets.vStretchLeft, 250f, panelBoxHeight, Vector2.zero);
            this.parentBrowser = parentBrowser;

        }
    }

    public class ResourceNameFolderInfoPanel: ResourceInfoPanelBase
    {
        private UIDynamicButton renameResourceButton;
        private GameObject fileNameTextGO;
        private UIDynamicTextField fileNameText;
        public ResourceNameFolderInfoPanel(GameObject parentViewportGO, float panelBoxHeight, BrowserUI parentBrowser) : base (parentViewportGO, panelBoxHeight, parentBrowser)
        {
            if (primaryRVGE != null)
            {
                float resourceFilenameBoxWidth = 245f;
                if (primaryRVGE.localFileResource && primaryRVGE.isRenameable)
                {
                    resourceFilenameBoxWidth = 195f;
                    renameResourceButton = UI.CreateUIButton(currentPanelColumnGO, 50f, 20f, "Rename", 20, 0, 0, AnchorPresets.topRight);
                    renameResourceButton.button.onClick.AddListener(delegate
                    {
                        renameResourceButton.gameObject.SetActive(false);
                        fileNameTextGO.SetActive(false);
                        fileNameText.gameObject.SetActive(true);
                        var inputComponent = fileNameText.gameObject.GetComponent<InputField>();
                        inputComponent.ActivateInputField();
                    });
                    parentBrowser.ApplyHelp(renameResourceButton, "Rename the resource file. This will change the actual filename and will be applicable outside of BrowserAssist, potentially breaking any dependencies on this file.");
                }
                fileNameTextGO = UI.CreateBasicTextBox(currentPanelColumnGO, primaryRVGE.resourceFileNameNoExtNoPrefix, resourceFilenameBoxWidth, 20f, 20, 0, 0, AnchorPresets.topLeft, backgoundColor: BrowserUI.lightBlueGrey);
                parentBrowser.ApplyHelp(fileNameTextGO, "Default Resource name is " + primaryRVGE.resourceFileNameNoExtNoPrefix + ". This is based on the actual filename of the resource.");

                var fileNameJSS = new JSONStorableString("fileNameJSS",primaryRVGE.resourceFileNameNoExtNoPrefix, FileNameUpdateCallback);
                fileNameText = UI.CreateTextBox(currentPanelColumnGO, fileNameJSS, 245f, 20f, 20, 0f, 0f, AnchorPresets.topLeft, true, InputField.LineType.SingleLine, "", 0, 0, null, TextAnchor.MiddleLeft, FileNameUpdateEndCallback);
                parentBrowser.ApplyHelp(fileNameText, "The resource Alias name is an alternative name that can be specified by the user and will override the default resource name. It is only used within BrowserAssist.");

                fileNameText.gameObject.SetActive(false);

                var aliasText = UI.CreateTextBox(currentPanelColumnGO, primaryRVGE.resourceNameAliasJSS, 245f, 20f, 20, 0f, -20f, AnchorPresets.topLeft, true, InputField.LineType.SingleLine, "Enter Alias for Resource Name", 0, 0,null,TextAnchor.MiddleLeft);
                parentBrowser.ApplyHelp(aliasText, "The resource Alias name is an alternative name that can be specified by the user and will override the default resource name. It is only used within BrowserAssist.");
                var clearAliasButton = UI.CreateUIButton(currentPanelColumnGO, 20f, 20f, "X", 20, -5f, -20f, AnchorPresets.topRight);
                parentBrowser.ApplyHelp(clearAliasButton, "Clear the alias name");
                clearAliasButton.button.onClick.AddListener(delegate { primaryRVGE.resourceNameAliasJSS.val = ""; });

                GameObject folderTextBoxGO;
                if (primaryRVGE.localFileResource)
                {
                    folderTextBoxGO=UI.CreateBasicTextBox(currentPanelColumnGO, new JSONStorableString("folder", primaryRVGE.folderPath), 215f, 20f, 20, 0, -40f, AnchorPresets.topLeft, true, backgoundColor: BrowserUI.lightBlueGrey);
                    var filterButton = UI.CreateUIButton(currentPanelColumnGO, 30f, 20f, "Filter", 20, 215f, -40f, AnchorPresets.topLeft, false);
                    parentBrowser.ApplyHelp(filterButton, "Apply this resource's folder as a Filter in the resource browser.");
                    filterButton.button.onClick.AddListener(delegate
                    {
                        parentBrowser.browserFiltersUI.FolderFilterSelected(primaryRVGE.containingLocalFolder);
                        parentBrowser.browserFiltersUI.creatorJSMultiEnum.valTopEnum = CreatorFilterChoices.localOnly;
                        
                    });

                }
                else folderTextBoxGO = UI.CreateBasicTextBox(currentPanelColumnGO, new JSONStorableString("folder", primaryRVGE.folderPath), 245f, 20f, 20, 0, -40f, AnchorPresets.topLeft,true, backgoundColor: BrowserUI.lightBlueGrey);
                parentBrowser.ApplyHelp(folderTextBoxGO, primaryRVGE.folderPath+"\nFolder that the resource is located within");

            }
            
        }
        private void FileNameUpdateEndCallback (string fileName)
        {
            renameResourceButton.gameObject.SetActive(true);
            fileNameTextGO.SetActive(true);
            fileNameText.gameObject.SetActive(false);
        }
        private void FileNameUpdateCallback(string newFileName)
        {
            renameResourceButton.gameObject.SetActive(true);
            fileNameTextGO.SetActive(true);
            fileNameText.gameObject.SetActive(false);

            char[] illegalChars = { '<', '>', ':', '"', '/','\\','|','?','*' };

            if (newFileName.IndexOfAny(illegalChars)==-1)
            {
                string newFileNameWithNoExt = newFileName;
                if (primaryRVGE.resourceCategory == ResourceCategory.preset) newFileNameWithNoExt = "Preset_" + newFileNameWithNoExt;
                string newFullPathFileNameNoExt = primaryRVGE.folderPath + "\\" + newFileNameWithNoExt;
                string newFullPathFileNameWithExt = newFullPathFileNameNoExt + "." + primaryRVGE.resourceFileNameExtension;
                if (!FileManagerSecure.FileExists(newFullPathFileNameWithExt))
                {
                    string sourceFolderPathFileName = Const.SymLinksFolder + "\\" + primaryRVGE.resourceFolderPathFileName;
                    string sourceFolderPathFileNameNoExt = Const.SymLinksFolder + "\\" + primaryRVGE.resourceFolderPathFileNameNoExt;
                    string targetFolderPathFileNameNoExt = Const.SymLinksFolder + "\\" + newFullPathFileNameNoExt;
                    string targetFolderPathFileName = Const.SymLinksFolder + "\\" + newFullPathFileNameWithExt;

                    try
                    {
                        FileManagerSecure.MoveFile(sourceFolderPathFileName, targetFolderPathFileName);
                    }
                    catch
                    {
                        SuperController.LogMessage("Move Selected Resources failed for '" + sourceFolderPathFileName + "'. Do you perhaps have the resource file open in a text editor?");
                        return;
                    }

                    try
                    {
                        if (FileManagerSecure.FileExists(sourceFolderPathFileName + ".fav"))
                        {
                            FileManagerSecure.MoveFile(sourceFolderPathFileName + ".fav", targetFolderPathFileName + ".fav");
                        }

                    }
                    catch
                    {
                        SuperController.LogMessage("Move Selected Resources Favourite Tag failed for '\"+ sourceFolderPathFileName+\"'.");
                    }

                    try
                    {
                        if (FileManagerSecure.FileExists(sourceFolderPathFileName + ".hide"))
                        {
                            FileManagerSecure.MoveFile(sourceFolderPathFileName + ".hide", targetFolderPathFileName + ".hide");
                        }

                    }
                    catch
                    {
                        SuperController.LogMessage("Move Selected Resources Hidden Tag failed for '\"+ sourceFolderPathFileName+\"'.");
                    }


                    try
                    {
                        if (FileManagerSecure.FileExists(sourceFolderPathFileNameNoExt + ".jpg"))
                        {
                            FileManagerSecure.MoveFile(sourceFolderPathFileNameNoExt + ".jpg", targetFolderPathFileNameNoExt + ".jpg");
                        }

                    }
                    catch
                    {
                        SuperController.LogMessage("Move Selected Resources Thumbnail image failed for '\"+ sourceFolderPathFileName+\"'.");
                    }

                    ResourceManifest.RenameRVGEFilename(primaryRVGE as LocalRVGE, newFullPathFileNameWithExt);
                    parentBrowser.resourceBrowserUI.UpdateDisplayName(primaryRVGE);
                    parentBrowser.resourceSelectionsUI.UpdateDisplayName(primaryRVGE);
                    parentBrowser.resourceActionsUI.RefreshUI();
                }
                else
                {
                    SuperController.LogMessage("BrowserAssist: The entered filename already exists in the folder. Chose an alternative filename.");
                }

            }
            else
            {
                SuperController.LogMessage("BrowserAssist: The entered filename contains illegal characters not allowed by Windows Operating System");
            }

        }
    }

    public class PackageInfoPanel : ResourceInfoPanelBase
    {
        public PackageInfoPanel(GameObject parentViewportGO, float panelBoxHeight, BrowserUI parentBrowser) : base(parentViewportGO, panelBoxHeight, parentBrowser)
        {
            if (primaryRVGE != null && !primaryRVGE.localFileResource)
            {
                PackagedRVGE packagedRVGE = primaryRVGE as PackagedRVGE;

                UI.CreateBasicTextBox(currentPanelColumnGO, "Creator:", 50f, 20f, 20, 0, 0, AnchorPresets.topLeft, backgoundColor: new Color(0,0,0,0));
                var textBox = UI.CreateBasicTextBox(currentPanelColumnGO,  packagedRVGE.creatorName, 165f, 20f, 20, 50f, 0, AnchorPresets.topLeft, backgoundColor: BrowserUI.lightBlueGrey);
                parentBrowser.ApplyHelp(textBox, "Name of the Creator who packaged the resource");

                var creatorFilterButton = UI.CreateUIButton(currentPanelColumnGO, 30f, 20f, "Filter", 20, 215f, 0f, AnchorPresets.topLeft, false);
                creatorFilterButton.button.onClick.AddListener(delegate { parentBrowser.browserFiltersUI.creatorJSMultiEnum.mainVal = primaryRVGE.creatorName; });
                parentBrowser.ApplyHelp(creatorFilterButton, "Update the filters so that only resources from this Creator are selectable");

                UI.CreateBasicTextBox(currentPanelColumnGO, "Package:", 50f, 20f, 20, 0, -20f, AnchorPresets.topLeft, backgoundColor: new Color(0, 0, 0, 0));
                textBox = UI.CreateBasicTextBox(currentPanelColumnGO, packagedRVGE.latestVARName, 195f, 20f, 20, 50f,-20f, AnchorPresets.topLeft, backgoundColor: BrowserUI.lightBlueGrey);
                parentBrowser.ApplyHelp(textBox, "Full package name that contains the resource");


                UI.CreateBasicTextBox(currentPanelColumnGO, "License:", 50f, 20f, 20, 0, -40f, AnchorPresets.topLeft, backgoundColor: new Color(0, 0, 0, 0));
                textBox = UI.CreateBasicTextBox(currentPanelColumnGO,  EnumManifest.GetDisplayVal(LicenseType.enumManifestName, packagedRVGE.varLicenseType), 165f, 20f, 20, 50, -40f, AnchorPresets.topLeft, backgoundColor: BrowserUI.lightBlueGrey);
                parentBrowser.ApplyHelp(textBox, "License Type that applies to package containing the resource");

                var licenseFilterButton = UI.CreateUIButton(currentPanelColumnGO, 30f, 20f, "Filter", 20, 215f, -40f, AnchorPresets.topLeft, false);
                licenseFilterButton.button.onClick.AddListener(delegate { parentBrowser.browserFiltersUI.licenseFilterJSEnum.val = packagedRVGE.varLicenseType; });
                parentBrowser.ApplyHelp(licenseFilterButton, "Update the filters so that only resources with this License are selectable");

            }

        }
    }

    public class ThumbnailInfoPanel : ResourceInfoPanelBase
    {
        public ThumbnailInfoPanel(GameObject parentViewportGO, float panelBoxHeight, BrowserUI parentBrowser) : base(parentViewportGO, panelBoxHeight, parentBrowser)
        {
            if (primaryRVGE != null)
            {
                var imageGO = UI.AddChildGOImage(currentPanelColumnGO,Color.white,AnchorPresets.topLeft,60f,60f,new Vector2(0,0),primaryRVGE.resourceSprite);
                if (FileManagerSecure.FileExists(FileManagerSecure.NormalizePath(primaryRVGE.resourceThumbnailReference))) parentBrowser.ApplyHelp(imageGO, primaryRVGE.resourceThumbnailReference);
                else parentBrowser.ApplyHelp(imageGO, "No thumbnail image exists for this resource.");

                if (!primaryRVGE.localFileResource || primaryRVGE.isRenameable)
                {
                    var button = UI.CreateUIButton(currentPanelColumnGO, 185f, 20f, "Take screenshot for Thumbnail...", 20, 60f, 0f, AnchorPresets.topLeft);
                    button.button.onClick.AddListener(delegate {
                        if (primaryRVGE.localFileResource)
                        {
                            SuperController.singleton.DoSaveScreenshot(Const.SymLinksFolder + "\\"+primaryRVGE.resourceFolderPathFileNameNoExt+".jpg",ResourceScreenShotCallback);
                            
                        }
                        else
                        {
                            PackagedRVGE rvge = primaryRVGE as PackagedRVGE;
                            
                            FileManagerSecure.CreateDirectory(rvge.customThumbnailFolder);

                            SuperController.singleton.DoSaveScreenshot(rvge.customThumbnailFile, ResourceScreenShotCallback);

                        }
                        
                    });
                    if (primaryRVGE.localFileResource)
                    {
                        parentBrowser.ApplyHelp(button, "Take a VAM screenshot to replace the Thumbnail image for this resource. The thumbnail will overwrite the original thumbnail and will be used by the VAM native File Browser.",true);
                    }
                    else
                    {
                        parentBrowser.ApplyHelp(button, "Take a VAM screenshot to use as a custom Thumbnail image for this resource. The custom thumbnail will not overwrite the original thumbnail and will only be used in BrowserAssist.");
                    }

                    button = UI.CreateUIButton(currentPanelColumnGO, 185f, 20f, "Select thumbnail image from disk...", 20, 60f, -20f, AnchorPresets.topLeft);
                    button.button.onClick.AddListener(delegate
                    {
                        SuperController.singleton.GetMediaPathDialog(ThumbnailFileCallback, "jpg","",false,true,showInstallFolderInDirectoryList:true);
                    });

                    if (primaryRVGE.localFileResource)
                    {
                        parentBrowser.ApplyHelp(button, "Select an image file from disk storage to replace the Thumbnail image for this resource. The thumbnail will overwrite the original thumbnail and will be used by the VAM native File Browser.", true);
                    }
                    else
                    {
                        parentBrowser.ApplyHelp(button, "Select an image from file system to use as a custom Thumbnail image for this resource. The custom thumbnail will not overwrite the original thumbnail and will only be used in BrowserAssist.");
                    }


                    if (!primaryRVGE.localFileResource)
                    {
                        PackagedRVGE rvge = primaryRVGE as PackagedRVGE;
                        if (FileManagerSecure.FileExists(rvge.customThumbnailFile))
                        {
                            button = UI.CreateUIButton(currentPanelColumnGO, 185f, 20f, "Reset thumbnail to default", 20, 60f, -40f, AnchorPresets.topLeft);
                            button.buttonColor = Color.red;
                            button.buttonText.color = Color.white;
                            parentBrowser.ApplyHelp(button, "Clear the custom Thumbnail image and revert to using the default resource thumbnail image supplied by the VAR creator. This will immediately and permanently delete the custom thumbnail.", true);
                            button.button.onClick.AddListener(delegate
                            {
                                FileManagerSecure.DeleteFile(rvge.customThumbnailFile);
                                primaryRVGE.ReloadSprite();
                            }
                            );
                        }

                    }
                    
                }


            }

        }

        private void ThumbnailFileCallback(string fileName)
        {
            if (fileName != "")
            {
                if (primaryRVGE.localFileResource)
                {
                    string targetThumbnail = Const.SymLinksFolder + "\\" + primaryRVGE.resourceFolderPathFileNameNoExt + ".jpg";
                    FileManagerSecure.DeleteFile(targetThumbnail);
                    FileManagerSecure.CopyFile(fileName, targetThumbnail);
                }
                else
                {
                    PackagedRVGE rvge = primaryRVGE as PackagedRVGE;

                    FileManagerSecure.CreateDirectory(rvge.customThumbnailFolder);
                    FileManagerSecure.DeleteFile(rvge.customThumbnailFile);
                    FileManagerSecure.CopyFile(fileName, rvge.customThumbnailFile);
                }
                primaryRVGE.ReloadSprite();
            }

        }
        private void ResourceScreenShotCallback(string text)
        {
          
            primaryRVGE.ReloadSprite();
            parentBrowser.SetActive();
        }
    }

    public class NotesInfoPanel : ResourceInfoPanelBase
    {
        public NotesInfoPanel(GameObject parentViewportGO, float panelBoxHeight, BrowserUI parentBrowser) : base(parentViewportGO, panelBoxHeight, parentBrowser)
        {
            if (primaryRVGE != null)
            {
                
                UI.CreateTextBox(currentPanelColumnGO, primaryRVGE.notesJSS, 245f, 60f, 20, 0f, 0f, AnchorPresets.topLeft, true, InputField.LineType.MultiLineSubmit, "Enter any notes related to this resource", 0, 10f, null, TextAnchor.UpperLeft);


            }

        }
    }



}
